/* $Id: agentmisc.C,v 1.8 1999/04/21 00:36:40 kaminsky Exp $ */

/*
 *
 * Copyright (C) 1998 David Mazieres (dm@uun.org)
 * Copyright (C) 1999 Michael Kaminsky (kaminsky@lcs.mit.edu)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "agentmisc.h"
#include "crypt.h"
#include "rxx.h"

str dotsfs; 
str agentsock;
ptr<aclnt> agentc;

void
usrdir_set ()
{
  sfsconst_init ();

  if (!dotsfs) {
    const char *home = getenv ("HOME");
    if (!home)
      fatal ("No HOME environment variable set\n");
    dotsfs = strbuf ("%s/.sfs", home);
  }
}

void
usrdir_agentsock ()
{
    usrdir_set ();

    if (!agentsock) {
      str name = myname ();
      if (!name)
	fatal ("could not find my host name\n");
      agentsock = dotsfs << "/" << name << ".sock";
    }
}

void
usrdir_make ()
{
  usrdir_set ();

  struct stat sb;
  if (stat (dotsfs, &sb) < 0) {
    if (mkdir (dotsfs, 0700) < 0 || stat (dotsfs, &sb) < 0)
      fatal << dotsfs << ": " << strerror (errno) << "\n";
    else
      warn << "Creating a " << dotsfs << " directory for you.\n";
  }
  if (!S_ISDIR (sb.st_mode))
    fatal << dotsfs << ": not a directory\n";
  if (sb.st_mode & 077)
    fatal ("%s: mode 0%o should be 0700\n",
	   dotsfs.cstr (), sb.st_mode & 0777);
}

inline bool
execok (const char *path)
{
  struct stat sb;
  return !stat (path, &sb) && S_ISREG (sb.st_mode) && (sb.st_mode & 0111);
}

static rxx colonplus (":+");
str
find_program (const char *program, const char *argv0)
{
  str r;
#ifdef MAINTAINER
  if (builddir) {
    r = fix_exec_path (program);
    if (execok (r))
      return r;
  }
#endif MAINTAINER
  const char *p;
  if (argv0 && (p = strrchr (argv0, '/'))) {
    r = str (argv0, p - argv0 + 1) << program;
    if (execok (r))
      return r;
  }
  p = getenv ("PATH");
  if (!p)
    return NULL;
  vector<str> vs;
  split (&vs, colonplus, p);
  for (str *sp = vs.base (); sp < vs.lim (); sp++) {
    if (!*sp || !sp->len ())
      continue;
    r = *sp << "/" << program;
    if (execok (r))
      return r;
  }
  return NULL;
}
