// -*-c++-*-
/* $Id: afsroot.h,v 1.11 1999/03/25 06:05:42 dm Exp $ */

/*
 *
 * Copyright (C) 1998 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

afsnode *automount (svccb *sbp, str name);
afsdir *userdir (const svccb *);

class afsroot : public afsdir {
protected:
  sfs_aid lastaid;

  afsroot () : afsdir (NULL) {}
  virtual ~afsroot () {}

public:
  void bumpmtime () { mtime.seconds++; }
  virtual afsnode *lookup (const str &name, const svccb *);
  virtual void nfs_lookup (svccb *, str name);

  virtual bool entryok (afsdirentry *, const svccb *);
  virtual afsdirentry *firstentry (const svccb *);
  virtual afsdirentry *nextentry (afsdirentry *, const svccb *);

  virtual void mkfattr3 (fattr3 *, const svccb *);

  static ptr<afsroot> alloc () { return New refcounted<afsroot>; }
};

const size_t maxulinks = 256;

class afsusrdir : public afsdir {
  afsroot *const root;
  size_t nentries;
public:
  bhash<str> negcache;

  void bumpmtime () { root->bumpmtime (); }
  afsusrdir (afsroot *r) : afsdir (r), root (r), nentries (0) {}
  void mkfh (nfs_fh *fhp) { root->mkfh (fhp); }
  bool chkfh (const nfs_fh *) { warn ("afsusrdir: chkfh\n"); return false; }

  bool link (afsnode *node, const str &name)
    { if (!afsdir::link (node, name)) return false; nentries++; return true; }
  bool unlink (const str &name)
    { if (!afsdir::unlink (name)) return false; nentries--; return true; }

  bool mkulink (const str &path, const str &name);
  void clrulink (const str &name);

  static ref<afsusrdir> alloc (afsroot *r)
    { return New refcounted<afsusrdir> (r); }
};

bool nameok (const str &name);
