/* $Id: srvcon.C,v 1.5 1999/05/16 08:37:09 dm Exp $ */

/*
 *
 * Copyright (C) 1999 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "sfsrwcd.h"
#include "axprt_crypt.h"

srvcon::srvcon (const ref<server> &ss, int fd, cb_t cb)
  : s (ss), cb (cb), kc (client_key)
{
  assert (s->locked ());

  s->x = NULL;
  s->sfsc = NULL;
  s->nfsc = NULL;
  s->sfscbs = NULL;
  s->nfscbs = NULL;
  s->clear ();

  if (fd < 0)
    getsock ();
  else {
    setfd (fd);
    crypt ();
  }
}

void
srvcon::fail (int err)
{
  s->x = NULL;
  s->sfsc = NULL;
  (*cb) (NULL, err);
  delete this;
}

void
srvcon::setfd (int fd)
{
  s->x = axprt_crypt::alloc (fd);
  s->sfsc = aclnt::alloc (s->x, sfs_program_1);
  s->nfsc = NULL;
  s->sfscbs = NULL;
  s->nfscbs = NULL;
}

void
srvcon::getsockres (int fd)
{
  if (fd < 0)
    fail (errno);
  else {
    setfd (fd);
    sfs_connectarg arg;
    arg.release = SFS_RELEASE;
    arg.service = SFS_SFS;
    arg.name = s->hostinfo.hostname;
    arg.hostid = s->hostid;
    s->sfsc->call (SFSPROC_CONNECT, &arg, &conres,
		   wrap (this, &srvcon::sfsconres));
  }
}

void
srvcon::sfsconres (clnt_stat err)
{
  if (err)
    fail (EIO);
  else if (conres.status)
    fail (conres.status);
  else if (!sfs_ckhostid (&s->hostid, conres.reply->host))
    fail (ENOENT);
  else
    crypt ();
}

void
srvcon::cryptres (const sfs_hash *sessid)
{
  if (sessid) {
    s->authinfo.sessid = *sessid;
    getfsinfo ();
  }
  else
    fail (EIO);
}

void
srvcon::getfsinfores (clnt_stat err)
{
  if (err)
    fail (EIO);
  else if (fsires.prog != ex_NFS_PROGRAM || fsires.nfs->vers != ex_NFS_V3)
    fail (EPROTONOSUPPORT);
  else {
    s->nfsc = aclnt::alloc (s->x, ex_nfs_program_3);
    s->sfscbs = asrv::alloc (s->x, sfscb_program_1,
			     wrap (&*s, &server::cbdispatch));
    s->nfscbs = asrv::alloc (s->x, ex_nfscb_program_3,
			     wrap (&*s, &server::cbdispatch));
    (*cb) (&fsires, 0);
    delete this;
  }
}
